import { defineProperty as _defineProperty, typeof as _typeof } from '../../_virtual/_rollupPluginBabelHelpers.js';

var TYPEOF_OBJECT = "object";
var TYPEOF_ARRAY = "array";
var TYPEOF_STRING = "string";
var TYPEOF_NUMBER = "number";
var TYPEOF_BOOLEAN = "boolean"; //

var PROP_PROPERTIES = "PROP_PROPERTIES";
var PROP_COPY_MISSING_FIELDS = "PROP_COPY_MISSING_FIELDS"; //fields which are missing in target map? true or false

var PROP_COPY_NULL_FIELDS = "PROP_COPY_NULL_FIELDS"; //if src field is empty, still copy ? true or false

var PROP_ONLY_SET_EMPTY_FIELDS = "PROP_ONLY_SET_EMPTY_FIELDS"; //copy from src to target only if target field is empty? true or false

var PROP_AUTO_MAP_SAME_NAME_FIELDS = "PROP_AUTO_MAP_SAME_NAME_FIELDS"; //map same name fields even if not defined in map? true or false

var PROP_FIELD_MAP = "PROP_FIELD_MAP";
var PROP_FIELD_NAME = "PROP_FIELD_NAME"; // DEFAULT MAPS

var PROPS_MAP_DEFAULT = {
  PROP_AUTO_MAP_SAME_NAME_FIELDS: true,
  PROP_COPY_NULL_FIELDS: true,
  PROP_ONLY_SET_EMPTY_FIELDS: false
};
var PROPS_MAP_MERGE = {
  PROP_AUTO_MAP_SAME_NAME_FIELDS: true,
  PROP_COPY_NULL_FIELDS: true,
  PROP_ONLY_SET_EMPTY_FIELDS: true
};
var PROPS_MAP_STRICT = {
  PROP_AUTO_MAP_SAME_NAME_FIELDS: false,
  PROP_COPY_NULL_FIELDS: true,
  PROP_ONLY_SET_EMPTY_FIELDS: false
}; // DEFAULT MAPS

var getTypeOfVar = function getTypeOfVar(variable) {
  if (variable) ; else {
    return "";
  }

  if (_typeof(variable) === TYPEOF_BOOLEAN) {
    return TYPEOF_BOOLEAN;
  } else if (_typeof(variable) === TYPEOF_STRING) {
    return TYPEOF_STRING;
  } else if (_typeof(variable) === TYPEOF_NUMBER) {
    return TYPEOF_NUMBER;
  } else if (_typeof(variable) === TYPEOF_OBJECT) {
    if (Array.isArray(variable)) {
      return TYPEOF_ARRAY;
    } else {
      return TYPEOF_OBJECT;
    }
  }
};

var copyProperties = function copyProperties(source, target, propertiesMap) {
  return copyPropertiesWithDepth(source, target, propertiesMap);
};

var copyPropertiesWithDepth = function copyPropertiesWithDepth(source, target, propertiesMap) {
  var trustProperties = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

  if (source && getTypeOfVar(source) === TYPEOF_OBJECT) ; else {
    return null;
  }

  if (target && getTypeOfVar(target) === TYPEOF_OBJECT) ; else {
    target = {};
  }

  var props = {};

  if (propertiesMap && propertiesMap[PROP_PROPERTIES]) {
    if (!trustProperties) {
      props = copyPropertiesWithDepth(PROPS_MAP_DEFAULT, propertiesMap[PROP_PROPERTIES], _defineProperty({}, PROP_PROPERTIES, PROPS_MAP_MERGE), true);
    } else {
      props = propertiesMap[PROP_PROPERTIES];
    }
  } else {
    props = PROPS_MAP_DEFAULT;
    propertiesMap = _defineProperty({}, PROP_PROPERTIES, PROPS_MAP_DEFAULT);
  } //make copy, for deep copy


  var src = JSON.parse(JSON.stringify(source));
  var tgt = JSON.parse(JSON.stringify(target));
  var fieldMap = {};

  if (propertiesMap[PROP_FIELD_MAP]) {
    fieldMap = propertiesMap[PROP_FIELD_MAP];

    if (getTypeOfVar(fieldMap) === TYPEOF_OBJECT) ; else {
      fieldMap = {};
    }
  }

  var fieldMapKeys = Object.keys(fieldMap);
  var srcKeys = Object.keys(src);

  for (var i = 0; i < srcKeys.length; i++) {
    var srcKey = srcKeys[i];
    var srcVal = src[srcKey];
    var isMappingAvailable = fieldMapKeys.includes(srcKey);
    var targetKey = "";
    var targetVal = tgt[srcKey];
    var mapToObjectProps = null;
    var mapToObjectPropsType = "normal"; //find field to map to & type (normal or Obj or Arr)

    if (srcVal || srcVal === false) ; else {
      if (!props[PROP_COPY_NULL_FIELDS]) {
        continue;
      }
    }

    if (targetVal || targetVal === false) {
      if (props[PROP_ONLY_SET_EMPTY_FIELDS]) {
        continue;
      }
    }

    if (isMappingAvailable) {
      mapToObjectProps = fieldMap[srcKey];

      if (getTypeOfVar(mapToObjectProps) === TYPEOF_OBJECT) {
        targetKey = mapToObjectProps[PROP_FIELD_NAME];
      } else if (getTypeOfVar(mapToObjectProps) === TYPEOF_STRING) {
        targetKey = mapToObjectProps;
      } else {
        targetKey = srcKey;
      }

      mapToObjectPropsType = getTypeOfVar(mapToObjectProps);
    } else {
      if (!props[PROP_AUTO_MAP_SAME_NAME_FIELDS]) {
        continue;
      }

      mapToObjectProps = propertiesMap;
      mapToObjectPropsType = getTypeOfVar(srcVal);
      targetKey = srcKey;
    }

    if (mapToObjectPropsType === TYPEOF_OBJECT) {
      if (getTypeOfVar(srcVal) === TYPEOF_OBJECT) {
        var objVal = copyPropertiesWithDepth(srcVal, {}, mapToObjectProps);
        tgt[targetKey] = objVal;
      } else if (getTypeOfVar(srcVal) === TYPEOF_ARRAY) {
        var arr = [];
        var arrayVal = null;

        for (var _i = 0; _i < srcVal.length; _i++) {
          if (getTypeOfVar(srcVal[_i]) === TYPEOF_OBJECT) {
            arrayVal = copyPropertiesWithDepth(srcVal[_i], {}, mapToObjectProps);
          } else {
            arrayVal = srcVal[_i];
          }

          arr.push(arrayVal);
        }

        tgt[targetKey] = arr;
      } else {
        continue;
      }
    } else {
      tgt[targetKey] = srcVal;
    }
  }

  return tgt;
};

export { PROPS_MAP_DEFAULT, PROPS_MAP_MERGE, PROPS_MAP_STRICT, PROP_AUTO_MAP_SAME_NAME_FIELDS, PROP_COPY_MISSING_FIELDS, PROP_COPY_NULL_FIELDS, PROP_FIELD_MAP, PROP_FIELD_NAME, PROP_ONLY_SET_EMPTY_FIELDS, PROP_PROPERTIES, copyProperties };
